/******************************************************************************/
// modul: threadCtrlDlg.h    version 1.22b (english)                          //
//                                                                            //
//    dialog with progress gauge, cancel/bkgnd- buttons and timer-powered     //
//		callback of a user specified function to provide display of actual      //
//		state.                                                                  //
//                                                                            //
//    written by lore@newty.de   -   home: www.newty.de                       //
//    copyright 2000 by lore                                                  //
//                                                                            //
//    FREE for all kinds of non-commercial use including modifying and        //
//    redistribution as long, as this header is not removed!                  //
//    disclaimer: nothing is guaranteed. you use it on your own risk.         //
//                                                                            //
//    please report bugs immediately to bugs@newty.de !!                      //
//                                                                            //
/******************************************************************************/


/***********************************************************************************************************/
// I. general
//      the thread-control dialog is used to control an addional thread. you start your thread-
//		  function and supply a pointer to an instance of this class in the argument list. in your
//		  thread-function you call setStopFlag(bool* flag) and supply the pointer to a flag. this
//		  flag is set when the user chooses to close the thread. so you have got to check this flag
//      periodically in your thread-function. call shutDown() as the last	statement in your
//      thread-function.
//		  note: ALL actions with the dialog including the try to close it only result in setting
//      the stop-flag. the dialog is closed using shutDown() (see above).
//
//  II. details
//    - you can execute the dialog modal or not modal with or without the buttons 'cancel' and 'background'.
//		  if executed modal, the button cancel and the 'x' in the titlebar result in return value ID_CANCEL
//		  and the button 'background' leads to return value ID_BKGND.
//      annotation: there is no sense in executing the dialog none modal with buttons :-)
//
//		- you supply the pointer to a callback function and its argument list using setCallbackFunc(...).
//		  your function must have the	signature 'void (void* argList)' and will be called periodically.
//
//		- you supply a stop-flag using setStopFlag(bool* flag) which is set if the user hits cancel/bkgnd or 'x'.
//
//		- if the user hits the 'x' is the titlebar (close window) this action results in the same actions as
//		  hed pressed the 'cancel'-button, i.g. the stop-flag is set but the dialog is not closed. the user
//		  is informed with a messagebox if no stop flag exists.
//
//		- you can set the status texts of the dialog using setText1() and setText2().
//    - the button 'background' can be disabled calling: enableBkgndButton(false). note: this functions
//		  needs a valid window-handle to work: thus call it after Create(). if you want to use it with
//		  modal dialogs: change the code :-)
//
/***********************************************************************************************************/


#ifndef _DIALOGS_H
#define _DIALOGS_H


#define ID_BKGND   105
#define ID_CANCEL	 106


#include <owl\owlpch.h>
#include <owl\button.h>			// due to: 	TButton
#include <owl\static.h>			// 			TStatic
#include <owl\gauge.h>			// 			TGauge



/***********************************************************************************************************/
// class: thread-control dialog
class TThreadCtrlDlg : public TDialog
{
public:

	// constructor/destructor
	TThreadCtrlDlg(TWindow* _parent, const char* _szTitle, TResId id, TFont* font, bool buttons=false);
   ~TThreadCtrlDlg();


   //******************************************************************************************
   // public functions:
   // 1.+2. set the text of the dialogs static-text fields
	// 3.    set the gauges value
   // 4.    supply the pointer to a (state-)function which is called periodically
   // 5. 	supply stop flag
   // 6. 	wrapper to close dialog
   void 	setText1(const char* szText);
   void 	setText2(const char* szText);
	void 	setGauge(const int value);
   void 	setCallbackFunc(void (*_callbackFunc)(void*), void* _argList, const int time = 1000);
   void  setStopFlag(bool* _stop) { stop = _stop;};
	void 	shutDown();

   void enableBkgndButton(const bool enable){ if(button2)button2->EnableWindow(enable);};

   int	progress;							// progress-counter

private:
   TWindow* parent;
	TFont*	font;

   TGauge	*gauge;							// progress gauge
   TButton 	*button1,*button2;			// the two buttons 'cancel' and 'background'
   TStatic	*text1, 	*text2;				// the two textfields


	// evaluate timer: if the user supplied a callback function -> call it
	void EvTimer(unsigned int id);

   void SetupWindow();
	void EvSysCommand(uint cmdType, TPoint& point);		// catch close command
	void EvCancel();                                  	// button 'cancel' or 'x' pressed
	void EvBkgnd();												// button 'background' pressed


   void (*callbackFunc)(void*);			// pointer to callback function
   void* argList;								// pointer to argument list of callback function

   bool* stop;									// pointer to stop-flag


   int time;									// timer intervall
   int idTimer;			 					// timer id
   int returnValue;							// keeps the return value - only used when executed modal



   // buffer for caption and status texts (see comments of setText1() in cpp-file)
   char szTitle[256];
  	char textBuffer1[256];
   char textBuffer2[256];


  	CRITICAL_SECTION synchro;	  // critical section to prevent multithreading side effects

   DECLARE_RESPONSE_TABLE(TThreadCtrlDlg);
};


#endif
