/******************************************************************************/
// modul: about.cpp   version 1.0b (english)                                  //
//                                                                            //
//    an about dialog with a few lines of text and an icon playing pinball    //
//		with the dialogs frame. by lore@newty.de                               //
//                                                                            //
//    see header-file for further information and disclaimer.                 //
//                                                                            //
/******************************************************************************/


#include <math.h>								// due to: 	cos() ...

#include "about.h"
#include "main.rh"							// due to the icon ids


#define ID_TIMER_SMILE 	1					// timer id
#define N_CHARS			55					// number of characters for the static textfields
#define SPEED				2.5				// "speed" of the moving icon (something like pixel per timer-event)


/***********************************************************************************************************/
// class TIconWnd: window with the icon which is moved along
TIconWnd::TIconWnd(TWindow* _parent, TIcon* _icon2Draw) : TWindow(_parent)
{
	icon2Draw = _icon2Draw;

   // set style and background color
   Attr.ExStyle |= WS_EX_TRANSPARENT;
   SetBkgndColor(GetSysColor(COLOR_MENU));
}



DEFINE_RESPONSE_TABLE1(TAbout, TWindow)
   EV_WM_CLOSE,
   EV_WM_TIMER,
END_RESPONSE_TABLE;




/***********************************************************************************************************/
// constructor
TAbout::TAbout(TWindow* _parent,  const char* title, const TFont* _font) : TWindow(_parent, title)
{
   font 		= _font;

 	// set style and background color
   Attr.Style 	= WS_POPUP | WS_VISIBLE | WS_CAPTION | WS_SYSMENU;
   SetBkgndColor(GetSysColor(COLOR_MENU));


 	// get fonts height and average width
	const int h = font->GetHeight();
   const int w = font->GetAveWidth();

   const int vOffset = 0.5*h;			// top and left margin,
   const int hOffset = 3*w;


   // calculate window size
   const int wWnd = w*N_CHARS + 2*hOffset;
   const int hWnd = 7*h       + 2*vOffset;

   // load icons, choose the one which is drawn and create icon-window
   icon1 		= new TIcon(GetModule()->LoadIcon(MAKEINTRESOURCE(IDI_SMILEY0)));
  	icon2 		= new TIcon(GetModule()->LoadIcon(MAKEINTRESOURCE(IDI_SMILEY1)));
   icon2Draw 	= icon2;
	smiley 		= new TIconWnd(this, icon2);



   // initialize startposition and moving direction
   xIcon 			= hOffset + w*25;
   yIcon 			= vOffset + 0.0*h;
  	directionPhi  	= - 1;
   nSmileTime 		= 1;


   // create static texts
   row1	= new TStatic(this, -1, "row1", hOffset, 	vOffset, w*N_CHARS, h);
   row2	= new TStatic(this, -1, "row2", hOffset, 	vOffset+1.5*h, w*N_CHARS, h);
   row3	= new TStatic(this, -1, "row3", hOffset, 	vOffset+4.0*h, w*N_CHARS, h);
   row4	= new TStatic(this, -1, "row4", hOffset, 	vOffset+5.5*h, w*N_CHARS, h);


   // center window and adjust its size
   int x = (GetSystemMetrics(SM_CXSCREEN)-wWnd)/2;
   int y = (GetSystemMetrics(SM_CYSCREEN)-hWnd)/2;

	TRect rc(x, y, x+wWnd, y+hWnd);
   AdjustWindowRect(rc, Attr.Style, false);
   MoveWindow(rc);

}


/***********************************************************************************************************/
// destructor
TAbout::~TAbout()
{
	KillTimer(ID_TIMER_SMILE);		// kill timer
   delete icon1;
   delete icon2;
}


/***********************************************************************************************************/
// setup window: all action which requires a valid window handle is done here
void TAbout::SetupWindow()
{
	TWindow::SetupWindow();


   // assign fonts
   row1		->SetWindowFont(HFONT (*font), false);
   row2		->SetWindowFont(HFONT (*font), false);
   row3		->SetWindowFont(HFONT (*font), false);
   row4		->SetWindowFont(HFONT (*font), false);

   // set timer for the icon animation. 55 ms (to be precise: 54.925ms) is the minimal possible time for win95
   SetTimer(ID_TIMER_SMILE, 55);

 	smiley->moveIcon(xIcon, yIcon);				// move icon
   smiley->drawIcon(icon2);						// and redraw it
}



/***********************************************************************************************************/
// functions sets the dialogs texts
void TAbout::setText(int row, const char* text)
{
	switch(row)	// "quick and dirty" :=)
   {
   	case 1 : row1->SetText(text); break;
   	case 2 : row2->SetText(text); break;
   	case 3 : row3->SetText(text); break;
   	case 4 : row4->SetText(text); break;
   }
}


/***********************************************************************************************************/
// evaluate timer: calculate the new icon position and check for contact with the dialogs frame
void TAbout::EvTimer(unsigned int id)
{
	if (id == ID_TIMER_SMILE)       // its not really necessary to check the id cause theres only one timer
   {
		// eventually reset the icon to the default (non-smiling) one
      if (nSmileTime == 0)
      	smiley->drawIcon(icon2);
      nSmileTime--;


      TRect rc = GetClientRect();			// get client rectangle
      xIcon +=  SPEED*cos(directionPhi);    // calculate new position
      yIcon += -SPEED*sin(directionPhi);


      // check for reflection at the dialogs frame
      if (xIcon < 0)																			// left border
      {
      	xIcon 			= 0;
      	directionPhi 	= -directionPhi + M_PI;
   		icon2Draw 		= icon1;															// set "smiling" icon
			nSmileTime 		=5;
			smiley->drawIcon(icon1);
      }


      if (yIcon < 0)																			// top border
      {
      	yIcon = 0;
			directionPhi 	= -directionPhi;
   		icon2Draw = icon1;            												// set "smiling" icon
			nSmileTime =5;
			smiley->drawIcon(icon1);
      }

      if (xIcon > rc.right-32)															// right border
      {
      	xIcon = rc.right-32;
         directionPhi 	= -directionPhi + M_PI;
   		icon2Draw = icon1;                                                // set "smiling" icon
			nSmileTime =5;
			smiley->drawIcon(icon1);
		}

      if (yIcon > rc.bottom-32)														  	// bottom border
      {
       	yIcon = rc.bottom-32;
			directionPhi 	= -directionPhi;
   		icon2Draw = icon1;                                                // set "smiling" icon
			nSmileTime =5;
			smiley->drawIcon(icon1);
      }

		smiley->moveIcon(xIcon, yIcon);				// move icon to new position
   }
}


