/******************************************************************************/
// modul: threadCtrlDlg.cpp    version 1.22b (english)                        //
//                                                                            //
//    dialog with progress gauge, cancel/bkgnd- buttons and timer-powered     //
//		callback of a user specified function. by lore@newty.de                 //
//    see header-file for further information and disclaimer.                 //
//                                                                            //
/******************************************************************************/



#include "threadCtrlDlg.h"

#include <windows.h>			// due to: 	CALLBACK
#include <stdio.h>			// 			sprintf()



/***********************************************************************************************************/
// this callback function is called from the timer created in SetupWindow(). it sends a WM_TIMER message to
// the thread-control dialog . this mechanism is used to be able to use the api-version of SetTimer()
VOID CALLBACK sendTimerMsg(HWND hwnd, UINT, UINT /* timerId */, DWORD )	{ SendMessage(hwnd, WM_TIMER, 0, 0); }



/***********************************************************************************************************/
// response table of thread-control dialog
DEFINE_RESPONSE_TABLE1(TThreadCtrlDlg, TDialog)
  	EV_BN_CLICKED(ID_CANCEL, 	EvCancel),
  	EV_BN_CLICKED(ID_BKGND, 	EvBkgnd),
   EV_WM_TIMER,
	EV_WM_SYSCOMMAND,			// prevent closing  when the user hits the 'x' in the dialogs titlebar
END_RESPONSE_TABLE;



/***********************************************************************************************************/
// constructor
TThreadCtrlDlg::TThreadCtrlDlg(TWindow* _parent, const char* _szTitle, TResId id, TFont* _font, bool buttons /* false */)
					: TDialog(_parent, id)
{
	// copy parameters
   parent 		= _parent;
   font			= _font;

   // initialize callback function, stop-flag and timer id. none of it exists yet.
   callbackFunc= NULL;
   stop			= NULL;
   idTimer		= -1;
   returnValue = ID_CANCEL;	// default return value - only used when executed modal
   progress		= 0;

	// "clear" text-buffers
   textBuffer1[0] = '\0';
   textBuffer2[0] = '\0';


   strcpy(szTitle, _szTitle);		// copy caption to set it in SetupWindow()


   // if needed: create buttons instead of 2nd static textfield
   if(buttons)
	{
	   button1 	= new TButton(this, ID_CANCEL, 	"Cancel", 		0, 0, 0, 0);
		button2 	= new TButton(this, ID_BKGND, 	"Background", 	0, 0, 0, 0);
      text2		= NULL;
   }
   else
   {
   	button1 	= NULL;
      button2 	= NULL;
      text2 	= new TStatic(this, -1, 0, 0, 0, 0, false);
   }
   text1 	= new TStatic(this, -1, 0, 0, 0, 0, false);


	// create gauge and enforce the borland version to be used. now its possible to use SetLed(), SetColor() etc.
	gauge		= new TGauge(this, "%d%%", -1, 0, 0, 0, 0);
   gauge->SetNativeUse(nuNever);
	gauge->SetLed(0,100);


   // critical section to synchronize some actions, exspecially when closing the dialog
   InitializeCriticalSection(&synchro);
}


/***********************************************************************************************************/
// destructor
TThreadCtrlDlg::~TThreadCtrlDlg()
{
	EnterCriticalSection(&synchro);
   LeaveCriticalSection(&synchro);
   DeleteCriticalSection(&synchro);								// clear critical section
}


/***********************************************************************************************************/
// setup all window elements. all action which need a valid window-handle is performed here.
void TThreadCtrlDlg::SetupWindow()
{
	TWindow::SetupWindow();		// call base classs version

 // a) set caption and background color
   SetCaption(szTitle);
   SetBkgndColor(GetSysColor(COLOR_SCROLLBAR ));



 // b) calculate window size according to the font supplied in constructor
   TRect rc = parent->GetWindowRect();					// get size of parents(!) client rectangle

   int h	 = font->GetHeight();				 			// get height and average width of the font
	int w  = font->GetAveWidth()*1.2;  					// thumb rule: multiply with 1.2

	int wWnd = 40*w;											// calculate needed size
   int hWnd = 5.5*h;

   int wPar =  rc.Width();									// offset to center dialog in the parent-window
   int hPar =  rc.Height();

   int left = rc.left+(wPar-wWnd)/2;					// calculate top left corner
   int top 	= rc.top+(hPar-hWnd)/2;

   TRect wRect(left, top, left+wWnd, top+hWnd);
   AdjustWindowRect(wRect, GetStyle(), false);     // adjust: size was calculated for the client rectangle

	MoveWindow(wRect, true);								// move


 // c) calculate position and move buttons, gauge and static textfields
   TRect rcc = GetClientRect();                    // get actual client rectangle (its easier to ask :-)


   int xText = (rcc.Width()-36*w)/2;					// left margin of the textfields

   // move buttons and/or static textfields
	if(button1)
   {
	   button1 ->MoveWindow((rcc.Width()-26*w)/2, 3.5*h, 10*w, 1.5*h, true);
   	button2 ->MoveWindow((rcc.Width()-26*w)/2+2*w+10*w, 3.5*h, 14*w, 1.5*h, true);

   	button1->SetWindowFont(*font, true);						// set font
   	button2->SetWindowFont(*font, true);
   }
   else
	{
		text2->MoveWindow(xText, 3.5*h, 36*w, h, true);
   	text2->SetWindowFont(*font, true);                    // set font
   }
   text1->MoveWindow(xText, 0.5*h, 36*w, h, true);
  	text1->SetWindowFont(*font, true);


	// move gauge and set range of it
  	gauge->MoveWindow(xText, 2*h, 36*w, h, true);
   gauge->SetRange(0, 100);
   setGauge(progress);			// set gauges value


 // d) if a callback function is supplied so far -> ... create timer and transfer buffertexts to the static
 //    textfields.
  	if(callbackFunc)	  				idTimer = ::SetTimer(GetHandle(), 0, time, sendTimerMsg);
   if(textBuffer1[0] != '\0')   	text1->SetText(textBuffer1);
   if(textBuffer2[0] != '\0')   	text2->SetText(textBuffer2);

}




/***********************************************************************************************************/
// function catches all messages with "sytem-commands". the message SC_CLOSE ("user clicked close ('x') ")
// is processed, all others are forwarded to the base class.
void TThreadCtrlDlg::EvSysCommand(uint cmdType, TPoint& point)
{
	if (cmdType == SC_CLOSE)		// catch close-window-command
   {
   	// if a stop flag was provided: ask user to set it, i.g. ask to stop
		if(stop)
		{
      	if(MessageBox("interrupt current process??","warning:", MB_YESNO | MB_ICONWARNING)== IDYES)
         	EvCancel();
      }
		// else: display error and do nothing
      else
			MessageBox("current process cannot be interrupted!!","error:", MB_OK | MB_ICONERROR);

   }
   else
   	TDialog::EvSysCommand(cmdType, point);		// forward all other messages to the base class
}




/***********************************************************************************************************/
// user pressed button 'cancel' or called via EvSysCommand() cause user pressed 'x' in titlebar
void TThreadCtrlDlg::EvCancel()
{
	// if a stop flag was supplied -> ...
	EnterCriticalSection(&synchro);
	if(stop)
		*stop = true;								// ... set it (never let this if-statement be interrupted)
   LeaveCriticalSection(&synchro);


   returnValue = ID_CANCEL;					// set return value which is used in shutDown()
}


/***********************************************************************************************************/
// user pressed button 'background'
void TThreadCtrlDlg::EvBkgnd()
{
	// if a stop-flag was supplied -> ...
	EnterCriticalSection(&synchro);
	if(stop)
		*stop = true;								// ... set it (never let this if-statement be interrupted)
   LeaveCriticalSection(&synchro);

   returnValue = ID_BKGND;						// set return value which is used in shutDown()
}



/***********************************************************************************************************/
// evaluate timer: call the user specified function
void TThreadCtrlDlg::EvTimer(unsigned int /* id */)
{
	EnterCriticalSection(&synchro);
	if(callbackFunc)
   	(*callbackFunc)(argList);	 			// call function using the function-pointer
   LeaveCriticalSection(&synchro);
}



/***********************************************************************************************************/
// set gauge value and change the color of the gauge according to the value
void TThreadCtrlDlg::setGauge(const int value)
{
	// calculate the actual zone
   int zone = value/25;

   // set the color according to the zone
   switch(zone)
   {
   	case 0 : gauge->SetColor(TColor(128, 0, 0));		break;	// 0-25%		: dark red
		case 1 :
		case 2 : gauge->SetColor(TColor(0, 0, 128));		break;	// 25-75%	: navy
		case 3 : gauge->SetColor(TColor(0, 128, 0));		break;	// 75-100%	: dark green
   }

 	gauge->SetValue(value);			// set the gauges value
}



/***********************************************************************************************************/
// wrapper to set the texts of the static textfields.
// there is a multi-threading problem: it may be, that these functions are called before SetupWindow() is
// executed. thus the call would have no effect! therefor the text is addionally copied to a buffer which is
// copied to the static textfield in SetupWindow(). annotation: you could also check the handles validity
// but you would have to synchronize with critical-sections here and(!) in SetupWindow().

void TThreadCtrlDlg::setText1(const char* text)
{
  	strcpy(textBuffer1, text);
  	text1->SetText(text);
}

void TThreadCtrlDlg::setText2(const char* text)
{
  	strcpy(textBuffer2, text);

	if(text2)
	  	text2->SetText(text);
}


/***********************************************************************************************************/
// use this function to supply a callback function. a timer is started and your function is called
// periodically. hint: the timer creation is done in SetupWindow() if the window handle is not valid yet.
void TThreadCtrlDlg::setCallbackFunc(void (*_callbackFunc)(void*), void* _argList, const int _time /* = 1000 */)
{
	callbackFunc = _callbackFunc;		// copy function-pointer and ...
   argList 		 = _argList;			// ... pointer to argument list


   // if timer is already running -> return
	if(idTimer != -1)
   	return;


   // else:
	time = _time;																	// remember time
   if(GetHandle() != NULL)		  												// if handle is valid -> ...
	   idTimer = ::SetTimer(GetHandle(), 0, time, sendTimerMsg);	// ... create timer
}



/***********************************************************************************************************/
// wrapper to close the dialog
void TThreadCtrlDlg::shutDown()
{
   // it may be, that the stop-flag is not valid in near future: set it to NULL
	// hint: for example a problem could arise if the mainthread is still hanging in EvSysCommand()s message-
   // box "interrupt current process??". if then the stop-flag gets deleted and afterwards you let EvCancel()
   // to be called when you leave the messagebox, 'stop' is an invalid pointer and its usage causes an
   // access-violation.
	EnterCriticalSection(&synchro);
	stop 			= NULL;
   callbackFunc= NULL;

  	// annotation: it can be, that the dialog is closed again before SetupWindow() is executed. thus check!
   if(idTimer != -1)
   	::KillTimer(GetHandle(), idTimer); 						// clear timer


   if(IsModal)
		while(!EndDialog(GetHandle(), returnValue));			// close modal dialog using EndDialog()
   else
	  	SendMessage(WM_CLOSE);										// close none modal one using a message


   LeaveCriticalSection(&synchro);
}


